// First include that we not write on our own
#include "stdarg.h"

// A temp char buffer
char _buffer[64];

// Returns the number representation of value "val" to the base "base".
static inline char *strfmtint(unsigned long val, int base)
{
	// We cannot have a base > 36
	if(base > 36)
		return "";
	
	// If the value is 0 the result is 0, too.
	if(val == 0)
		return "0";
	
	const char	*digits = "0123456789abcdefghijklmnopqrstuvwxyz";
	char		*p = _buffer + 64;
	
	int i = 0;
	*p = '\0';
	
	// Make each char of "val" fit into the base representation
	// we want. For example, a pointer (represented by a hex) has
	// a base of 16 and can therefore only be represented by 0-16 of
	// the char array.
	while(val)
	{
		*--p = digits[val % base];
		val /= base;
		
		i++;
	}
	
	return p;
}

// Takes the format char array and browses it for "%"s which
// indicates that we have to replace the following char by
// a value from the argument list.
// The final string is the "str" parameter whereas the result
// is the number of written parameters.
int vsprintf(char *str, const char *format, va_list arg)
{
	int written = 0;
	
	// Iterate through the format char array
	while(*format != '\0')
	{
		// If we hit a "%" which is the indicator for a replacement
		if(*format == '%')
		{
			format ++;
			
			switch(*format)
			{
				// Is our parameter a char?
				case 'c':
				{
					char temp = va_arg(arg, int);
					str[written] = temp;
				}
					break;
					
				case 's':
				{
					// Is our parameter a string?
					char *string = va_arg(arg, char *);
					
					while(*string != '\0')
					{
						str[written] = *string;
						
						written ++;
						string ++;
					}
					
					str[written] = ' ';
					written --;
				}
					break;
					
					
				case 'i':
				{
					// Is our parameter an integer?
					unsigned long	temp = va_arg(arg, unsigned long);
					char			*string = strfmtint(temp, 10);
					
					while(*string != '\0')
					{
						str[written] = *string;
						
						written ++;
						string ++;
					}
					
					str[written] = ' ';
					written --;
				}
					break;
					
					
				case 'p':
				{
					// Is our parameter a pointer? //Return a hexadecimal representation
					unsigned long	temp = (unsigned long)va_arg(arg, void *);
					char			*string = strfmtint(temp, 16);
					
					str[written] = '0';
					written ++;
					
					str[written] = 'x';
					written ++;
					
					while(*string != '\0')
					{
						str[written] = *string;
						
						written ++;
						string ++;
					}
					
					str[written] = ' ';
					written --;
				}
					break;
					
				case 'x':
				{
					// Is our parameter a XX?
					unsigned long	temp = va_arg(arg, unsigned long);
					char			*string = strfmtint(temp, 16);
					
					while(*string != '\0')
					{
						str[written] = *string;
						
						written ++;
						string ++;
					}
					
					str[written] = ' ';
					written --;
					
				}
					break;
					
				case '%':
					// Or do we really want to write a "%"?
					str[written] = '%';					
					break;
					
				default:
					// Otherwise print the text that is given in the input string.
					str[written] = '%';
					written ++;
					
					str[written] = *format;
					break;
			}
		}
		else 
		{
			str[written] = *format;
		}
		
		
		written ++;
		format ++;
	}
	
	str[written] = '\0';	
	// Return the numbers of written parameters
	return written;
}

// Convers a format char array to a char array
int sprintf(char *dst, const char *format, ...)
{
	// Create an argument list
	va_list param;
	// Put our arguments (format) into this argument list
	va_start(param, format);
	
	// Converts the format string to a string
	int written = vsprintf(dst, format, param);
	
	// Close the argument list
	va_end(param);
	
	// Return the number of parameters written
	return written;
}

// Our final function to simply put out a char array
// with arguments.
void putfs(const char *format, ...)
{
	// Once again read arguments, convert, ...
	// but this time we use "puts" to outpur the
	// string to our console.
	va_list param;
	va_start(param, format);
	
	char temp[1024];
	vsprintf(temp, format, param);
	
	puts(temp);
	va_end(param);
}

