#include "system.h"

extern unsigned long end;

unsigned int *page_aligned_end = 0;

//unsigned long *page_directory = (unsigned long)&end;
unsigned long *page_directory = (unsigned long *)0x9C000;
unsigned long *page_table = (unsigned long *) 0x9D000;

unsigned long address=0; // holds the physical address of where a page is
unsigned int i;

void page_fault(struct regs *r)
{

   // A page fault has occurred.
   // The faulting address is stored in the CR2 register.
   u32int faulting_address;
   asm volatile("mov %%cr2, %0" : "=r" (faulting_address));

   // The error code gives us details of what happened.
   int present   = !(r->err_code & 0x1); // Page not present
   int rw = r->err_code & 0x2;           // Write operation?
   int us = r->err_code & 0x4;           // Processor was in user-mode?
   int reserved = r->err_code & 0x8;     // Overwritten CPU-reserved bits of page entry?
   int id = r->err_code & 0x10;          // Caused by an instruction fetch?

   // Output an error message.
   puts("Page fault! ( ");
   if (present) {puts("present ");}
   if (rw) {puts("read-only ");}
   if (us) {puts("user-mode ");}
   if (reserved) {puts("reserved ");}
   puts(") at ");
   putfs("%p", faulting_address);
   puts("\n");
}

void initPaging() {

	// map the first 4MB of memory
	for(i=0; i<1024; i++)
	{
		page_table[i] = address | 3; // attribute set to: supervisor level, read/write, present(011 in binary)
		address = address + 4096; // 4096 = 4kb
	};

	// fill the first entry of the page directory
	page_directory[0] = page_table; // attribute set to: supervisor level, read/write, present(011 in binary)
	page_directory[0] = page_directory[0] | 3;

	for(i=1; i<1024; i++)
	{
		page_directory[i] = 0 | 2; // attribute set to: supervisor level, read/write, not present(010 in binary)
	};

	asm volatile("mov %0, %%cr3":: "r"(page_directory));
    unsigned long cr0;
    asm volatile("mov %%cr0, %0": "=r"(cr0));
	putfs("CR0 value is: %i\n",cr0);
	cr0 |= 0x80000000;
    asm volatile("mov %0, %%cr0":: "r"(cr0));
}
// go celebrate or something 'cause PAGING IS ENABLED!!!!!!!!!!
