; This is our entry point. As mentioned in this tutorial we write this code in assembler
; since it is easier to define our stack here and to mark this file as bootable binary for
; GRUB.

; We are moving in a 32 bits environment. 
[BITS 32]
global start
start:
    mov esp, _sys_stack     ; This points the stack to our new stack area
    jmp stublet

; This part MUST be 4byte aligned so that GRUB can read the magic number (boot signature),
; the flags and the checksum.
ALIGN 4

; This part is to make our kernel GRUB compatible and tell grub some flags, for
; instance that we use the AOUT binary format instead of ELF. This is neccessary
; because we use DJGPP under Windows.
mboot:
    ; Multiboot macros to make a few lines later more readable
    MULTIBOOT_PAGE_ALIGN	equ 1<<0
    MULTIBOOT_MEMORY_INFO	equ 1<<1
    MULTIBOOT_AOUT_KLUDGE	equ 1<<16
    MULTIBOOT_HEADER_MAGIC	equ 0x1BADB002
    MULTIBOOT_HEADER_FLAGS	equ MULTIBOOT_PAGE_ALIGN | MULTIBOOT_MEMORY_INFO | MULTIBOOT_AOUT_KLUDGE
    MULTIBOOT_CHECKSUM	equ -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	; Extern tells the compiler that the label can be found in another module.
    EXTERN code, bss, end

    dd MULTIBOOT_HEADER_MAGIC
    dd MULTIBOOT_HEADER_FLAGS
    dd MULTIBOOT_CHECKSUM
    
    ; When we build our kernel the assembler fills these values automatically.
	; For mboot the address of the "mboot" label is set. This is to show GRUB
	; where it can find which section.
    dd mboot
    dd code
    dd bss
    dd end
    dd start

; Everything is defined properly. Now we can start our main loop in assembler.
stublet:
	extern _main
	call _main
    jmp $


; Place holder for GDT code.


; Place holder for interrupt code.


; The bss section holds data that is not jet defined. For example, a "dd"
; would be ignored by the compiler.
; Now we define our stack of 8KB. Remember that a stack actually grows
; downwards, so we declare the size of the data before declaring
; the identifier '_sys_stack'
SECTION .bss
    resb 8192 ;reserve 8KB
_sys_stack: