#include <system.h>

// Local variables for text pointer, background and foreground color
// and for the cursor coordinates
unsigned short *textmemptr;
int attrib = 0x0F;
int csr_x = 0, csr_y = 0;

// Function to scroll the screen for one line if needed.
void scroll(void)
{
    unsigned blank, temp;

    // We define a blank line.
    blank = 0x20 | (attrib << 8);

    // If we find ourselves at the last line we need to scoll.
    if(csr_y >= 25)
    {
        // We move the entire text on the screen up one line.
        temp = csr_y - 25 + 1;
        memcpy (textmemptr, textmemptr + temp * 80, (25 - temp) * 80 * 2);

        // We set the last line to our blank line.
        memsetw (textmemptr + (25 - temp) * 80, blank, 80);
        csr_y = 25 - 1;
    }
}

// Updates the blinking cursor. This is done by the VGA adapter
// as you can see on the functions "outportb" where we write
// to an external IO Port.
void move_csr(void)
{
    unsigned curpos;

    // Find the position where the cursor has to be placed.
    curpos = csr_y * 80 + csr_x;

	// Here we write the position to the Control Register of
	// the VGA controller.
    outportb(0x3D4, 14);
    outportb(0x3D5, curpos >> 8);
    outportb(0x3D4, 15);
    outportb(0x3D5, curpos);
}

// Clear the screen
void cls()
{
    unsigned blank;
    int i;

    // Like the scrolling function we define a blank line.
    blank = 0x20 | (attrib << 8);

    // Now we put this blank line in all 25 lines of our screen.
    for(i = 0; i < 25; i++)
        memsetw (textmemptr + i * 80, blank, 80);

    // Set the cursor to 0/0 and set the blinking cursor there.
    csr_x = 0;
    csr_y = 0;
    move_csr();
}

void delchar()
{
	unsigned short *where;
	unsigned att = attrib << 8;
	char c = ' ';
	
	csr_x --;
	
	if(csr_x < 0)
	{
		csr_x = 0;
		csr_y --;
		
		if(csr_y < 0)
			csr_y = 0;
	}
	
	where = textmemptr + (csr_y * 80 + csr_x);
	*where = c | att;
	move_csr();
}


// Print a char on the screen.
void putch(unsigned char c)
{
    unsigned short *where;
    unsigned att = attrib << 8;

    // Backspace -> Move the cursor back
    if(c == 0x08)
    {
        if(csr_x != 0) csr_x--;
    }
	// Moves the cursor foreward to a point that can be divided by 8
    else if(c == 0x09)
    {
        csr_x = (csr_x + 8) & ~(8 - 1);
    }
	// Carriage Return -> Brings the cursor back to the beginning
	// of the current line
    else if(c == '\r')
    {
        csr_x = 0;
    }
    // New line -> increments the y value and sets the x value to 0.
    else if(c == '\n')
    {
        csr_x = 0;
        csr_y++;
    }
    // Everything else greater / equal than a space can be printed out.
	// Character and color will be printed out.
    else if(c >= ' ')
    {
        where = textmemptr + (csr_y * 80 + csr_x);
        *where = c | att;
        csr_x++;
    }

    // If we have printed out more than 79 characters
	// we add a new line.
    if(csr_x >= 80)
    {
        csr_x = 0;
        csr_y++;
    }

    // Scroll the screen and move the cursor.
    scroll();
    move_csr();
}

// Puts out a string which is now easy using putch
void puts(unsigned char *text)
{
    int i;

    for (i = 0; i < strlen(text); i++)
    {
        putch(text[i]);
    }
}

// Sets the foreground and background color
void settextcolor(unsigned char forecolor, unsigned char backcolor)
{
	// First 4 bytes for background, last 4 bytes for foreground.
    attrib = (backcolor << 4) | (forecolor & 0x0F);
}

// Initialize our video driver
void init_video(void)
{
	// @0xB8000 our video memory begins that holds the information
	// on the content of our screen.
    textmemptr = (unsigned short *)0xB8000;
    cls();
}