#include <system.h>

// For how many ticks has our system been running?
int timer_ticks = 0;

// Set the timer frequency
void timer_phase(int hz)
{
    int divisor = 1193180 / hz;
    outportb(0x43, 0x36); // Write the command we assembled
						  // in the tutorial to the command register
    outportb(0x40, divisor & 0xFF); // Set the high byte of the frequency
    outportb(0x40, divisor >> 8); // Set the low byte of the frequency
}

// This one handles the interrupt for the timer. We increment
// the timer_ticks everytime the interrupt is fired. By
// default the timer ticks 18222 times per second.
void timer_handler(struct regs *r)
{
    timer_ticks++;

	// Every 18 ticks (~1 second) we can execute an action here.
    if (timer_ticks % 18 == 0)
    {
        //puts("One second has passed\n");
    }
}

int nDummy = 0;
void dummy() {
	nDummy = nDummy + 1;
}

// This is a wait function which loops until a special
// time was waited for.
void timer_wait(int ticks)
{
    unsigned long eticks;

    eticks = timer_ticks + ticks;
    while(timer_ticks < eticks)
	{
		dummy();
	}
}

// We set up the system clock for IRQ0 to register to our
// timer.
void timer_install()
{
    irq_install_handler(0, timer_handler);
}